local AceGUI = LibStub("AceGUI-3.0")

local L = Capitalist.Locale
local realm = GetRealmName()
local character = UnitName("player")

local countPerPageProductUI = 13

local grayText = L["needing scan"]
local greenText = L["at cheapest price"]
local yellowText = L["needing repost"]
local orangeText = L["not worth listing"]
local blueText = L["with more room for profit"]
local redText = L["out of stock"]

local function CreateStatusLabel(frameParent)
	return frameParent:CreateFontString(nil, nil, "GameFontNormal")
end

local function CreateStatusButton(frameParent, fnOnClick)
	local button = CreateFrame("button", nil, frameParent, "UIPanelButtonTemplate")
	button:SetWidth(250)
	button:SetHeight(15)
	button:SetNormalTexture(nil)
	button:SetDisabledTexture(nil)
	button:SetPushedTexture(nil)
	button:SetScript("OnClick", fnOnClick)
	return button
end

local function SetButtonTextWithColor(button, text, color, disabled)
	local disable = disabled or false

	if (disable) then
		color = nil
		button:Disable()
	else
		button:Enable()
	end

	if (color == "gray") then
		button:SetText("|cffc0c0c0"..text.."|r")
	elseif (color == "green") then
		button:SetText("|cff00ff00"..text.."|r")
	elseif (color == "yellow") then
		button:SetText("|cffffff00"..text.."|r")
	elseif (color == "orange") then
		button:SetText("|cffff8000"..text.."|r")
	elseif (color == "blue") then
		button:SetText("|cff8080ff"..text.."|r")
	elseif (color == "red") then
		button:SetText("|cffff0000"..text.."|r")
	else
		button:SetText("|cff303030"..text.."|r")
	end

	button:SetWidth(button:GetTextWidth())
end

local function SetLabelColor(label, color, dimmed)
	local dim = dimmed or false

	if (dim) then
		color = nil
	end

	if (color == "gray") then
		label:SetTextColor(0.75, 0.75, 0.75)
	elseif (color == "green") then
		label:SetTextColor(0, 1, 0)
	elseif (color == "yellow") then
		label:SetTextColor(1, 1, 0)
	elseif (color == "orange") then
		label:SetTextColor(1, 0.5, 0)
	elseif (color == "blue") then
		label:SetTextColor(0.5, 0.5, 1)
	elseif (color == "red") then
		label:SetTextColor(1, 0, 0)
	else
		label:SetTextColor(0.2, 0.2, 0.2)
	end
end

local function SetLabelTextWithColor(label, text, color, dimmed)
	SetLabelColor(label, color, dimmed)
	label:SetText(text)
end

function Capitalist:CreateAuctionUI()
	self:DebugEntry("CreateAuctionUI")

	self.auctionUIButton = CreateFrame("button", nil, AuctionFrameBrowse)
	local t = self.auctionUIButton
	t:SetPoint("TOPLEFT", AuctionFrame, "TOPLEFT", 20, -70)
	t:SetHeight(30)
	t:SetWidth(30)
	t:SetScript("OnClick", function() Capitalist:ToggleAuctionUI() end)
	t:Show()
	
	t.texture = t:CreateTexture()
	t.texture:SetAllPoints()
	t.texture:SetTexture("Interface\\Icons\\INV_Scroll_03")
	t.texture:SetDrawLayer("BACKGROUND")
	
	t.border = t:CreateTexture()
	t.border:SetAllPoints()
	t.border:SetTexture("Interface\\Buttons\\ButtonHilight-Square")
	t.border:SetBlendMode("ADD")
	t.border:SetDrawLayer("BORDER")
	t.border:Hide()
	t:SetScript("OnEnter", function(self) self.border:Show() end)
	t:SetScript("OnLeave", function(self) if (not Capitalist.auctionUI:IsVisible()) then self.border:Hide() end end)
	
	
	self.auctionUI = CreateFrame("frame", nil, AuctionFrameBrowse)
	t = self.auctionUI
	t:SetPoint("TOPLEFT", AuctionFrame, "TOPLEFT", 16, -101)
	t:SetPoint("BOTTOMRIGHT", AuctionFrame, "BOTTOMRIGHT", -7, 37)
	t:SetFrameStrata("HIGH")
	t:Hide()
	
	t.background = t:CreateTexture(nil, "BACKGROUND")
	t.background:SetAllPoints()
	t.background:SetTexture(0,0,0)
	
	t.lastScanLabel = t:CreateFontString(nil, nil, "GameFontNormal")
	t.lastScanLabel:SetText(L["Last Scan: "].."-")
	t.lastScanLabel:SetPoint("TOPLEFT", 10, -10)
	t.lastScanLabel:SetWidth(110)
	t.lastScanLabel:SetHeight(20)

	t.progress = t:CreateTexture()
	t.progress:SetPoint("LEFT", t.lastScanLabel, "LEFT")
	t.progress:SetWidth(110)
	t.progress:SetHeight(20)
	t.progress:SetTexture(0, 0, 1)
	t.progress:Hide()
	
	t.progressLabel = t:CreateFontString(nil, nil, "GameFontNormal")
	t.progressLabel:SetAllPoints(t.lastScanLabel)
	
	t.scanButton = CreateFrame("button", nil, t, "UIPanelButtonTemplate")
	t.scanButton:SetWidth(110)
	t.scanButton:SetHeight(20)
	t.scanButton:SetPoint("TOPLEFT", t.lastScanLabel, "BOTTOMLEFT", 0, -5)
	t.scanButton:SetText(L["perform scan"])
	t.scanButton:SetScript("OnClick", function() Capitalist:StartScan() end)

	t.cancelButton = CreateFrame("button", nil, t, "UIPanelButtonTemplate")
	t.cancelButton:SetWidth(110)
	t.cancelButton:SetHeight(20)
	t.cancelButton:SetPoint("TOPLEFT", t.scanButton, "BOTTOMLEFT", 0, -5)
	t.cancelButton:SetText(L["cancel auctions"])
	t.cancelButton:SetScript("OnClick", function() Capitalist:CancelAuctions() end)
	t.cancelButton:Disable()

	t.postButton = CreateFrame("button", nil, t, "UIPanelButtonTemplate")
	t.postButton:SetWidth(110)
	t.postButton:SetHeight(20)
	t.postButton:SetPoint("TOPLEFT", t.cancelButton, "BOTTOMLEFT", 0, -5)
	t.postButton:SetText(L["post products"])
	t.postButton:SetScript("OnClick", function() Capitalist:StartPost() end)
	t.postButton:Disable()
	
	t.items = {}
	for i = 1, countPerPageProductUI do
		t.items[i] = CreateFrame("frame", nil, t)
		local item = t.items[i]
		item:SetWidth(500)
		item:SetHeight(20)
		item:SetPoint("TOPLEFT", t, "TOPLEFT", 220, -20*i)
		
		item.texture = item:CreateTexture()
		item.texture:SetBlendMode("ADD")
		item.texture:SetDrawLayer("BACKGROUND")
		item.texture:SetPoint("TOPLEFT", item, "TOPLEFT", 0, 0)
		item.texture:SetWidth(16)
		item.texture:SetHeight(16)
		item.texture:SetTexture("")

		item.name = item:CreateFontString(nil, nil, "GameFontNormal")
		item.name:SetWidth(230)
		item.name:SetPoint("LEFT", item.texture, "RIGHT", 4, 0)
		item.name:SetText("")
		item.name:SetJustifyH("LEFT")
		
		item.money = item:CreateFontString(nil, nil, "GameFontNormal")
		item.money:SetWidth(150)
		item.money:SetPoint("LEFT", item.name, "RIGHT")
		item.money:SetText("")
		item.money:SetJustifyH("LEFT")
		
		item.count = item:CreateFontString(nil, nil, "GameFontNormal")
		item.count:SetPoint("LEFT", item.money, "RIGHT")
		item.count:SetWidth(70)
		item.count:SetText("")
		
		item.lowest = item:CreateFontString(nil, nil, "GameFontNormal")
		item.lowest:SetPoint("LEFT", item.count, "RIGHT")
		item.lowest:SetWidth(100)
		item.lowest:SetText("")
	end
	
	t.page = 0
	
	t.prevButton = CreateFrame("button", nil, t, "UIPanelButtonTemplate")
	t.prevButton:SetWidth(30)
	t.prevButton:SetHeight(20)
	t.prevButton:SetText("<<")
	t.prevButton:SetPoint("TOPLEFT", t.items[countPerPageProductUI], "BOTTOMLEFT")
	t.prevButton:SetScript("OnClick", function()
										local i = Capitalist.auctionUI.page
										if (i > 0) then i = i - 1 end
										Capitalist.auctionUI.page = i
										Capitalist:UpdateAuctionUI()
									  end)
	t.prevButton:Disable()
	
	t.nextButton = CreateFrame("button", nil, t, "UIPanelButtonTemplate")
	t.nextButton:SetWidth(30)
	t.nextButton:SetHeight(20)
	t.nextButton:SetText(">>")
	t.nextButton:SetPoint("LEFT", t.prevButton, "RIGHT", 10, 0)
	t.nextButton:SetScript("OnClick", function()
									Capitalist.auctionUI.page = Capitalist.auctionUI.page + 1
									Capitalist:UpdateAuctionUI()
								  end)
	t.nextButton:Disable()
	
	t.cancelYellow = CreateFrame("CheckButton", nil, t, "UICheckButtonTemplate")
	t.cancelYellow:SetPoint("TOPLEFT", t.postButton, "BOTTOMLEFT", 0, -10)
	t.cancelYellow:SetScale(0.6)
	t.cancelYellow:SetChecked(self.db.profile.cancelYellow)
	t.cancelYellow:SetScript("OnClick", function(self)
											Capitalist.db.profile.cancelYellow = self:GetChecked() == 1
											Capitalist:CheckForCancel()
										end)
	
	t.cancelYellowLabel = t:CreateFontString(nil, nil, "GameFontNormal")
	t.cancelYellowLabel:SetPoint("LEFT", t.cancelYellow, "RIGHT")
	t.cancelYellowLabel:SetJustifyH("LEFT")
	t.cancelYellowLabel:SetText(L["cancel yellow auctions"])

	t.cancelBlue = CreateFrame("CheckButton", nil, t, "UICheckButtonTemplate")
	t.cancelBlue:SetPoint("TOPLEFT", t.cancelYellow, "BOTTOMLEFT", 0, 5)
	t.cancelBlue:SetScale(0.6)
	t.cancelBlue:SetChecked(self.db.profile.cancelBlue)
	t.cancelBlue:SetScript("OnClick", function(self)
											Capitalist.db.profile.cancelBlue = self:GetChecked() == 1
											Capitalist:CheckForCancel()
										end)
	
	t.cancelBlueLabel = t:CreateFontString(nil, nil, "GameFontNormal")
	t.cancelBlueLabel:SetPoint("LEFT", t.cancelBlue, "RIGHT")
	t.cancelBlueLabel:SetJustifyH("LEFT")
	t.cancelBlueLabel:SetText(L["cancel blue auctions"])

	t.cancelDuration = CreateFrame("CheckButton", nil, t, "UICheckButtonTemplate")
	t.cancelDuration:SetPoint("TOPLEFT", t.cancelBlue, "BOTTOMLEFT", 0, 5)
	t.cancelDuration:SetScale(0.6)
	t.cancelDuration:SetChecked(self.db.profile.cancelDuration)
	t.cancelDuration:SetScript("OnClick", function(self)
											Capitalist.db.profile.cancelDuration = self:GetChecked() == 1
											Capitalist:CheckForCancel()
										end)
	
	t.cancelDurationLabel = t:CreateFontString(nil, nil, "GameFontNormal")
	t.cancelDurationLabel:SetPoint("LEFT", t.cancelDuration, "RIGHT")
	t.cancelDurationLabel:SetJustifyH("LEFT")
	t.cancelDurationLabel:SetText(L["cancel auctions <= 12 hour"])

	t.grayButton = CreateStatusButton(t, function() Capitalist:OnClickStatus(self, "gray", grayText) end)
	t.greenButton = CreateStatusButton(t, function() Capitalist:OnClickStatus(self, "green", greenText) end)
	t.yellowButton = CreateStatusButton(t, function() Capitalist:OnClickStatus(self, "yellow", yellowText) end)
	t.orangeButton = CreateStatusButton(t, function() Capitalist:OnClickStatus(self, "orange", orangeText) end)
	t.blueButton = CreateStatusButton(t, function() Capitalist:OnClickStatus(self, "blue", blueText) end)
	t.redButton = CreateStatusButton(t, function() Capitalist:OnClickStatus(self, "red", redText) end)

	-- Note that these are positioned from the bottom up (dependencies are forwardly expressed)
	t.grayButton:SetPoint("BOTTOMLEFT", t.greenButton, "TOPLEFT", 0, 0)
	t.greenButton:SetPoint("BOTTOMLEFT", t.yellowButton, "TOPLEFT", 0, 0)
	t.yellowButton:SetPoint("BOTTOMLEFT", t.orangeButton, "TOPLEFT", 0, 0)
	t.orangeButton:SetPoint("BOTTOMLEFT", t.blueButton, "TOPLEFT", 0, 0)
	t.blueButton:SetPoint("BOTTOMLEFT", t.redButton, "TOPLEFT", 0, 0)
	t.redButton:SetPoint("BOTTOMLEFT", t, "BOTTOMLEFT", 40, 10)

	t.grayLabel = CreateStatusLabel(t)
	t.greenLabel = CreateStatusLabel(t)
	t.yellowLabel = CreateStatusLabel(t)
	t.orangeLabel = CreateStatusLabel(t)
	t.blueLabel = CreateStatusLabel(t)
	t.redLabel = CreateStatusLabel(t)

	t.grayLabel:SetPoint("RIGHT", t.grayButton, "LEFT", -5, 0)
	t.greenLabel:SetPoint("RIGHT", t.greenButton, "LEFT", -5, 0)
	t.yellowLabel:SetPoint("RIGHT", t.yellowButton, "LEFT", -5, 0)
	t.orangeLabel:SetPoint("RIGHT", t.orangeButton, "LEFT", -5, 0)
	t.blueLabel:SetPoint("RIGHT", t.blueButton, "LEFT", -5, 0)
	t.redLabel:SetPoint("RIGHT", t.redButton, "LEFT", -5, 0)

	self:DebugExit("CreateAuctionUI")
end

function Capitalist:ToggleAuctionUI()
	self:DebugEntry("ToggleAuctionUI")

	if (self.componentUI:IsVisible()) then
		self:ToggleComponentUI()
	end

	if (self.auctionUI:IsVisible()) then
		self.auctionUI:EnableMouse(false)
		self.auctionUI:Hide()
		self.auctionUIButton.border:Hide()
		
		BrowseQualitySort:Show()
		BrowseLevelSort:Show()
		BrowseDurationSort:Show()
		BrowseHighBidderSort:Show()
		BrowseCurrentBidSort:Show()
	else
		self.auctionUI:Show()
		self.auctionUI:EnableMouse(true)
		self.auctionUIButton.border:Show()
		
		BrowseQualitySort:Hide()
		BrowseLevelSort:Hide()
		BrowseDurationSort:Hide()
		BrowseHighBidderSort:Hide()
		BrowseCurrentBidSort:Hide()
	end

	self:DebugExit("ToggleAuctionUI")
end

function Capitalist:IdentifySkillColorAndOwner(product, skillName, productsInAH)
	-- needing scan
	local color = "gray"
	local owner

	local profession = self:FindProfession(skillName)
	local productName = self:ProductNameFromSkillName(profession, skillName)

	local v = self.list[productName]
	if (self.validScan) then
		if (productsInAH[productName]) then

			self:EnsureBestPrice(productName)
				
			if (v.buyoutSelf == nil) then
				color = "gray"
			elseif ((v.buyoutOthers == nil) or (v.buyoutSelf < v.buyoutOthers)) then
				-- at cheapest price
				color = "green"
				owner = character
			elseif (v.buyoutOthers < v.bestPrice) then
				-- not worth listing
				color = "orange"
				owner = v.otherOwner
			elseif (v.buyoutSelf < v.bestPrice) then
				-- with more room for profit
				color[skillName] = "blue"
				owner = character
			elseif (v.bestPrice < v.buyoutSelf) then
				-- needing repost
				color = "yellow"
				owner = v.otherOwner
			end
		else
			-- out of stock
			color = "red"
			if (v) then
				owner = v.otherOwner
			end
		end
	end

	return color, owner
end

function Capitalist:AuctionHouseProductListClear()
	self:DebugEntry("AuctionHouseProductListClear")
	self.auctionProductList = nil
	self:DebugExit("AuctionHouseProductListClear")
end

function Capitalist:AuctionHouseProductListEnsure()
	self:DebugEntry("AuctionHouseProductListEnsure")

	if (not self.auctionProductList) then
		self.auctionProductList = {}

		if (self.db.profile.professions) then
			for profession, professionData in pairs(self.db.profile.professions) do
				for skillName, skillData in pairs(professionData) do
					if (self:ProductIsVisible(profession, skillName) and
						self:ProductIsEnabled(profession, skillName) and
						(self:ProductGetCount(profession, skillName) > 0)) then

						local auctionProductListEntry = { profession = profession, skillName = skillName, }
						table.insert(self.auctionProductList, auctionProductListEntry)
					end
				end
			end

			table.sort(self.auctionProductList, function(a, b)
													return L.AuctionUIFromSkillName(a.profession, a.skillName) < L.AuctionUIFromSkillName(b.profession, b.skillName)
												end
						)
		end
	end

	self:DebugExit("AuctionHouseProductListEnsure")
end

function Capitalist:UpdateAuctionUI()
	self:DebugEntry("UpdateAuctionUI")

	local skillColor = {}
	local skillLowestOwner = {}
	local countOfColor = {gray = 0, green = 0, yellow = 0, orange = 0, blue = 0, red = 0}	
	
	if (self.db.profile.professions and
		self.db.profile[realm] and
		self.db.profile[realm][character]) then

		-- save, which products are on the AH (for restocking)
		local productsInAH = self.db.profile.storage[realm][character].AH
		for k, v in pairs(productsInAH) do
			productsInAH[k] = nil
		end

		for i = 1, GetNumAuctionItems("owner") do
			local name, _, _, _, _, _, _, _, buyoutPrice, _, _, _, saleStatus = GetAuctionItemInfo("owner", i)

			-- REVIEW: Correction for two glyphs named "Glyph of Death Coil"
			if (name == "Glyph of Death Coil") then
				local auctionLink = GetAuctionItemLink("owner", i)
				if (auctionLink) then
					local itemIdString = string.match(auctionLink, "Hitem:(%d+)")
					local itemIdNumber = tonumber(itemIdString)
			
					-- item 45804, spell 64266 - DK
					if (itemIdNumber == 45804) then
						name = name.." <Death Knight>"
					end
					-- item 42457, spell 57261 - Warlock
					if (itemIdNumber == 42457) then
						name = name.." <Warlock>"
					end
				end
			end

			if (self.list[name] and saleStatus == 0) then
				if (not productsInAH[name]) then
					productsInAH[name] = 1
				else
					productsInAH[name] = productsInAH[name] + 1
				end
			end
		end

		self:AuctionHouseProductListEnsure()

		for i=1, #self.auctionProductList do
			local auctionProductListEntry = self.auctionProductList[i]
			local profession = auctionProductListEntry.profession
			local skillName = auctionProductListEntry.skillName
			local product = self.db.profile.professions[profession][skillName]

			local color, owner = self:IdentifySkillColorAndOwner(product, skillName, productsInAH)
			skillColor[skillName] = color
			skillLowestOwner[skillName] = owner
			countOfColor[color] = countOfColor[color] + 1
		end
	
		-- check if we need to show previous/next-buttons
		local numPages = #self.auctionProductList / countPerPageProductUI
	
		if (numPages < self.auctionUI.page) then
			self.auctionUI.page = numPages
		end
	
		if (self.auctionUI.page > 0) then
			self.auctionUI.prevButton:Enable()
		else
			self.auctionUI.prevButton:Disable()
		end
	
		if (self.auctionUI.page < numPages - 1) then
			self.auctionUI.nextButton:Enable()
		else
			self.auctionUI.nextButton:Disable()
		end
	
		local item = 0 - self.auctionUI.page * countPerPageProductUI
	
		for index, auctionProductListEntry in ipairs(self.auctionProductList) do
			local skillName = auctionProductListEntry.skillName
			item = item + 1

			local profession = self:FindProfession(skillName)
			if (item > 0 and item <= countPerPageProductUI) then
				local product = self.db.profile.professions[profession][skillName]

				local productName = self:ProductNameFromSkillName(profession, skillName)

				local texture = self.auctionUI.items[item].texture
				local text = self.auctionUI.items[item].name
				local money = self.auctionUI.items[item].money
				local count = self.auctionUI.items[item].count
				local lowest = self.auctionUI.items[item].lowest

				texture:SetTexture("")
				if (product.itemLink) then
					local textureName = "Interface\\Icons\\INV_Scroll_03"
					if (product.itemLink:find("|Hitem")) then
						local idString = string.match(product.itemLink, "|H[^:]*:(%d+)")
						local idNumber = tonumber(idString)
						textureName = GetItemIcon(idNumber)
					end
					texture:SetTexture(textureName)
				end

				text:SetText(L.AuctionUIFromSkillName(profession, skillName))
			
				if (skillColor[skillName] ~= "gray" and skillColor[skillName] ~= "red") then
					self:EnsureProductCost(productName)

					local price = self:GetMoneyText(self.list[productName].buyoutSelf)
					local profit = self:GetMoneyText(self.list[productName].buyoutSelf - self.list[productName].cost, true)
				
					if (self.list[productName].buyoutOthers) then
						money:SetText(price.." ["..profit.."; "..self:GetMoneyText(self.list[productName].buyoutOthers - self.list[productName].buyoutSelf, true).."]")
					else
						money:SetText(price.." ["..profit.."]")
					end
				else
					money:SetText("")
				end
			
				local ownAuctions = productsInAH[productName] or 0
				if (self.validScan and
					self.list[productName] and
					self.list[productName].otherAuctions) then
					count:SetText("[ "..ownAuctions.." | "..#self.list[productName].otherAuctions.." ]")
				else
					count:SetText("[ "..ownAuctions.." ]")
				end

				lowest:SetText(skillLowestOwner[skillName])
			
				SetLabelColor(text, skillColor[skillName])
				SetLabelColor(count, skillColor[skillName])
				SetLabelColor(money, skillColor[skillName])
				SetLabelColor(lowest, skillColor[skillName])
			end 
		end
	
		if (item > 0 and item < countPerPageProductUI) then
			for i = item + 1, countPerPageProductUI do
				self.auctionUI.items[i].texture:SetTexture("")
				self.auctionUI.items[i].name:SetText("")
				self.auctionUI.items[i].count:SetText("")
				self.auctionUI.items[i].money:SetText("")
				self.auctionUI.items[i].lowest:SetText("")
			end
		end
	end
	
	SetButtonTextWithColor(self.auctionUI.grayButton, grayText, "gray", (countOfColor.gray == 0))
	SetButtonTextWithColor(self.auctionUI.greenButton, greenText, "green", (countOfColor.green == 0))
	SetButtonTextWithColor(self.auctionUI.yellowButton, yellowText, "yellow", (countOfColor.yellow == 0))
	SetButtonTextWithColor(self.auctionUI.orangeButton, orangeText, "orange", (countOfColor.orange == 0))
	SetButtonTextWithColor(self.auctionUI.blueButton, blueText, "blue", (countOfColor.blue == 0))
	SetButtonTextWithColor(self.auctionUI.redButton, redText, "red", (countOfColor.red == 0))

	SetLabelTextWithColor(self.auctionUI.grayLabel, countOfColor.gray.."x", "gray", (countOfColor.gray == 0))
	SetLabelTextWithColor(self.auctionUI.greenLabel, countOfColor.green.."x", "green", (countOfColor.green == 0))
	SetLabelTextWithColor(self.auctionUI.yellowLabel, countOfColor.yellow.."x", "yellow", (countOfColor.yellow == 0))
	SetLabelTextWithColor(self.auctionUI.orangeLabel, countOfColor.orange.."x", "orange", (countOfColor.orange == 0))
	SetLabelTextWithColor(self.auctionUI.blueLabel, countOfColor.blue.."x", "blue", (countOfColor.blue == 0))
	SetLabelTextWithColor(self.auctionUI.redLabel, countOfColor.red.."x", "red", (countOfColor.red == 0))

	self:DebugExit("UpdateAuctionUI")
end

function Capitalist:GetMoneyText(amount, sign, significantDigits)
	self:DebugEntry("GetMoneyText")

	if (amount == nil) then
		return ""
	end
	
	local str
	
	if (abs(amount) > 10000) then
		str = "g"
		amount = amount / 10000
	elseif (abs(amount) > 100) then
		str = "s"
		amount = amount / 100
	else
		str = "c"
	end
	
	if (not significantDigits) then
		amount = math.floor(amount + 0.5)
	else
		local factor = math.pow(10, significantDigits - 1)
		amount = math.floor(amount * factor) / factor
	end
	
	
	str = amount .. str
	
	if (sign and amount > 0) then
		str = "+"..str
	end
	
	return self:DebugExit("GetMoneyText", str)
end

function Capitalist:OnClickStatus(self, colorFilter, category)
	local self = Capitalist
	local productsInAH = Capitalist.db.profile.storage[realm][character].AH

	local outputtedHeader

		for profession, professionData in pairs(self.db.profile.professions) do
			self:DebugMisc(profession)

			for skillName, skillData in pairs(professionData) do
				local product = self.db.profile.professions[profession][skillName]
				local productName = self:ProductNameFromSkillName(profession, skillName)

				if (productName and
					self:ProductIsVisible(profession, skillName) and
					self:ProductIsEnabled(profession, skillName) and
					(self:ProductGetCount(profession, skillName) > 0)) then

					local color, owner = self:IdentifySkillColorAndOwner(product, skillName, productsInAH)
					if (color == colorFilter) then
						local skillText = skillName
						if (self.db.profile.professions and
							self.db.profile.professions[profession] and
							self.db.profile.professions[profession][skillName] and
							self.db.profile.professions[profession][skillName].itemLink) then
							skillText = self:SkillTextFromSkillName(profession, skillName)
						end

						if (not outputtedHeader) then
							local headerText = string.format(L["Products %s:"], category)
							self:Print(headerText)
							outputtedHeader = true
						end
						ChatFrame1:AddMessage("  "..skillText)
					end
				end
			end
		end

end
